@@ -241,7 +241,7 @@ xs_str *not_really_markdown(const char *content, xs_list **attach, xs_list **tag
 
             /* is it an URL to an image? */
             if (xs_startswith(v, "https:/" "/") && xs_startswith((t = xs_mime_by_ext(v)), "image/")) {
-                if (tag) {
+                if (tag && xs_str_in(s, k) != -1) {
                     /* add the emoji to the tag list */
                     xs *e = xs_dict_new();
                     xs *i = xs_dict_new();
@@ -311,9 +311,9 @@ xs_str *sanitize(const char *content)
 
                 s = xs_str_cat(s, s2);
             } else {
-                /* else? just show it with encoded code.. that's it. */
-                xs *el = encode_html(v);
-                s = xs_str_cat(s, el);
+                /* treat end of divs as paragraph breaks */
+                if (strcmp(v, "</div>"))
+                    s = xs_str_cat(s, "<p>");
             }
         }
         else {
