@@ -169,14 +169,13 @@ int server_get_handler(xs_dict *req, const char *q_path,
 {
     int status = 0;
 
-    (void)req;
-
     /* is it the server root? */
     if (*q_path == '\0') {
         xs_dict *q_vars = xs_dict_get(req, "q_vars");
         char *t = NULL;
 
         if (xs_type(q_vars) == XSTYPE_DICT && (t = xs_dict_get(q_vars, "t"))) {
+            /** search by tag **/
             int skip = 0;
             int show = xs_number_get(xs_dict_get(srv_config, "max_timeline_entries"));
             char *v;
@@ -194,12 +193,25 @@ int server_get_handler(xs_dict *req, const char *q_path,
                 more = 1;
             }
 
-            *body = html_timeline(NULL, tl, 0, skip, show, more, t);
+            char *accept = xs_dict_get(req, "accept");
+            if (!xs_is_null(accept) && strcmp(accept, "application/rss+xml") == 0) {
+                xs *link = xs_fmt("%s/?t=%s", srv_baseurl, t);
+
+                *body = timeline_to_rss(NULL, tl, link, link, link);
+                *ctype = "application/rss+xml; charset=utf-8";
+            }
+            else {
+                xs *page = xs_fmt("?t=%s", t);
+                xs *title = xs_fmt(L("Search results for tag #%s"), t);
+                *body = html_timeline(NULL, tl, 0, skip, show, more, title, page, 0);
+            }
         }
         else
         if (xs_type(xs_dict_get(srv_config, "show_instance_timeline")) == XSTYPE_TRUE) {
+            /** instance timeline **/
             xs *tl = timeline_instance_list(0, 30);
-            *body = html_timeline(NULL, tl, 0, 0, 0, 0, NULL);
+            *body = html_timeline(NULL, tl, 0, 0, 0, 0,
+                L("Recent posts by users in this instance"), NULL, 0);
         }
         else
             *body = greeting_html();
@@ -223,6 +235,15 @@ int server_get_handler(xs_dict *req, const char *q_path,
             srv_baseurl);
     }
     else
+    if (strcmp(q_path, "/.well-known/host-meta") == 0) {
+        status = 200;
+        *ctype = "application/xrd+xml";
+        *body  = xs_fmt("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+                "<XRD>"
+                "<Link rel=\"lrdd\" type=\"application/xrd+xml\" template=\"https://%s/.well-known/webfinger?resource={uri}\"/>"
+                "</XRD>", xs_dict_get(srv_config, "host"));
+    }
+    else
     if (strcmp(q_path, "/nodeinfo_2_0") == 0) {
         status = 200;
         *ctype = "application/json; charset=utf-8";
@@ -349,7 +370,7 @@ void httpd_connection(FILE *f)
 #ifndef NO_MASTODON_API
         if (status == 0)
             status = mastoapi_delete_handler(req, q_path,
-                    &body, &b_size, &ctype);
+                    payload, p_size, &body, &b_size, &ctype);
 #endif
     }
 
@@ -377,6 +398,7 @@ void httpd_connection(FILE *f)
                                 body, xs_dict_get(srv_config, "host"));
 
         headers = xs_dict_append(headers, "WWW-Authenticate", www_auth);
+        headers = xs_dict_append(headers, "Cache-Control", "no-cache, must-revalidate, max-age=0");
     }
 
     if (ctype == NULL)
@@ -392,7 +414,8 @@ void httpd_connection(FILE *f)
     xs_dict *more_headers = xs_dict_get(srv_config, "http_headers");
     if (xs_type(more_headers) == XSTYPE_DICT) {
         char *k, *v;
-        while (xs_dict_iter(&more_headers, &k, &v))
+        int c = 0;
+        while (xs_dict_next(more_headers, &k, &v, &c))
             headers = xs_dict_set(headers, k, v);
     }
 
@@ -710,13 +733,6 @@ void httpd(void)
     xs *shm_name = NULL;
     sem_t anon_job_sem;
 
-    /* setup the server stat structure */
-    p_state = srv_state_op(&shm_name, 0);
-
-    p_state->srv_start_time = time(NULL);
-
-    p_state->use_fcgi = xs_type(xs_dict_get(srv_config, "fastcgi")) == XSTYPE_TRUE;
-
     address = xs_dict_get(srv_config, "address");
     port    = xs_number_str(xs_dict_get(srv_config, "port"));
 
@@ -725,6 +741,13 @@ void httpd(void)
         return;
     }
 
+    /* setup the server stat structure */
+    p_state = srv_state_op(&shm_name, 0);
+
+    p_state->srv_start_time = time(NULL);
+
+    p_state->use_fcgi = xs_type(xs_dict_get(srv_config, "fastcgi")) == XSTYPE_TRUE;
+
     p_state->srv_running = 1;
 
     signal(SIGPIPE, SIG_IGN);
@@ -802,7 +825,7 @@ void httpd(void)
 
     /* send as many exit jobs as working threads */
     for (n = 1; n < p_state->n_threads; n++)
-        job_post(xs_stock_false, 0);
+        job_post(xs_stock(XSTYPE_FALSE), 0);
 
     /* wait for all the threads to exit */
     for (n = 0; n < p_state->n_threads; n++)
