@@ -1,7 +1,7 @@
 /* snac - A simple, minimalistic ActivityPub instance */
 /* copyright (c) 2022 - 2024 grunfink et al. / MIT license */
 
-#define VERSION "2.45"
+#define VERSION "2.53"
 
 #define USER_AGENT "snac/" VERSION
 
@@ -16,6 +16,10 @@
 #define MAX_THREADS 256
 #endif
 
+#ifndef MAX_CONVERSATION_LEVELS
+#define MAX_CONVERSATION_LEVELS 48
+#endif
+
 extern double disk_layout;
 extern xs_str *srv_basedir;
 extern xs_dict *srv_config;
@@ -25,6 +29,8 @@ extern int dbglevel;
 
 #define L(s) (s)
 
+#define POSTLIKE_OBJECT_TYPE "Note|Question|Page|Article|Video|Event"
+
 int mkdirx(const char *pathname);
 
 int valid_status(int status);
@@ -41,6 +47,7 @@ typedef struct {
     xs_dict *config;    /* user configuration */
     xs_dict *config_o;  /* user configuration admin override */
     xs_dict *key;       /* keypair */
+    xs_dict *links;     /* validated links */
     xs_str *actor;      /* actor url */
     xs_str *md5;        /* actor url md5 */
 } snac;
@@ -105,6 +112,10 @@ int object_del(const char *id);
 int object_del_if_unref(const char *id);
 double object_ctime_by_md5(const char *md5);
 double object_ctime(const char *id);
+double object_mtime_by_md5(const char *md5);
+double object_mtime(const char *id);
+void object_touch(const char *id);
+
 int object_admire(const char *id, const char *actor, int like);
 int object_unadmire(const char *id, const char *actor, int like);
 
@@ -132,7 +143,7 @@ int timeline_del(snac *snac, char *id);
 xs_list *timeline_simple_list(snac *snac, const char *idx_name, int skip, int show);
 xs_list *timeline_list(snac *snac, const char *idx_name, int skip, int show);
 int timeline_add(snac *snac, const char *id, const xs_dict *o_msg);
-void timeline_admire(snac *snac, const char *id, const char *admirer, int like);
+int timeline_admire(snac *snac, const char *id, const char *admirer, int like);
 
 xs_list *timeline_top_level(snac *snac, xs_list *list);
 xs_list *local_list(snac *snac, int max);
@@ -165,8 +176,14 @@ int is_hidden(snac *snac, const char *id);
 void tag_index(const char *id, const xs_dict *obj);
 xs_list *tag_search(char *tag, int skip, int show);
 
+xs_val *list_maint(snac *user, const char *list, int op);
+xs_list *list_timeline(snac *user, const char *list, int skip, int show);
+xs_val *list_content(snac *user, const char *list_id, const char *actor_md5, int op);
+void list_distribute(snac *user, const char *who, const xs_dict *post);
+
 int actor_add(const char *actor, xs_dict *msg);
 int actor_get(const char *actor, xs_dict **data);
+int actor_get_refresh(snac *user, const char *actor, xs_dict **data);
 
 int static_get(snac *snac, const char *id, xs_val **data, int *size, const char *inm, xs_str **etag);
 void static_put(snac *snac, const char *id, const char *data, int size);
@@ -187,7 +204,8 @@ xs_str *notify_check_time(snac *snac, int reset);
 void notify_add(snac *snac, const char *type, const char *utype,
                 const char *actor, const char *objid);
 xs_dict *notify_get(snac *snac, const char *id);
-xs_list *notify_list(snac *snac, int new_only);
+int notify_new_num(snac *snac);
+xs_list *notify_list(snac *snac, int skip, int show);
 void notify_clear(snac *snac);
 
 void inbox_add(const char *inbox);
@@ -198,6 +216,10 @@ int is_instance_blocked(const char *instance);
 int instance_block(const char *instance);
 int instance_unblock(const char *instance);
 
+int content_match(const char *file, const xs_dict *msg);
+xs_list *content_search(snac *user, const char *regex,
+            int priv, int skip, int show, int max_secs, int *timeout);
+
 void enqueue_input(snac *snac, const xs_dict *msg, const xs_dict *req, int retries);
 void enqueue_shared_input(const xs_dict *msg, const xs_dict *req, int retries);
 void enqueue_output_raw(const char *keyid, const char *seckey,
@@ -209,6 +231,9 @@ void enqueue_telegram(const xs_str *msg, const char *bot, const char *chat_id);
 void enqueue_ntfy(const xs_str *msg, const char *ntfy_server, const char *ntfy_token);
 void enqueue_message(snac *snac, const xs_dict *msg);
 void enqueue_close_question(snac *user, const char *id, int end_secs);
+void enqueue_object_request(snac *user, const char *id, int forward_secs);
+void enqueue_verify_links(snac *user);
+void enqueue_actor_refresh(snac *user, const char *actor, int forward_secs);
 void enqueue_request_replies(snac *user, const char *id);
 int was_question_voted(snac *user, const char *id);
 
@@ -245,7 +270,11 @@ const char *default_avatar_base64(void);
 
 xs_str *process_tags(snac *snac, const char *content, xs_list **tag);
 
+char *get_atto(const xs_dict *msg);
+xs_list *get_attachments(const xs_dict *msg);
+
 xs_dict *msg_admiration(snac *snac, char *object, char *type);
+xs_dict *msg_repulsion(snac *user, char *id, char *type);
 xs_dict *msg_create(snac *snac, const xs_dict *object);
 xs_dict *msg_follow(snac *snac, const char *actor);
 
@@ -273,6 +302,7 @@ xs_str *get_actor_inbox(const char *actor);
 int send_to_actor(snac *snac, const char *actor, const xs_dict *msg,
                   xs_val **payload, int *p_size, int timeout);
 int is_msg_public(const xs_dict *msg);
+int is_msg_from_private_user(const xs_dict *msg);
 int is_msg_for_me(snac *snac, const xs_dict *msg);
 
 int process_user_queue(snac *snac);
@@ -285,18 +315,21 @@ int activitypub_post_handler(const xs_dict *req, const char *q_path,
                              char *payload, int p_size,
                              char **body, int *b_size, char **ctype);
 
-xs_str *not_really_markdown(const char *content, xs_list **attach);
+xs_dict *emojis(void);
+xs_str *not_really_markdown(const char *content, xs_list **attach, xs_list **tag);
 xs_str *sanitize(const char *content);
 xs_str *encode_html(const char *str);
 
-xs_str *html_timeline(snac *user, const xs_list *list, int local,
-                      int skip, int show, int show_more, char *tag);
+xs_str *html_timeline(snac *user, const xs_list *list, int read_only,
+                      int skip, int show, int show_more,
+                      char *title, char *page, int utl);
 
 int html_get_handler(const xs_dict *req, const char *q_path,
                      char **body, int *b_size, char **ctype, xs_str **etag);
 int html_post_handler(const xs_dict *req, const char *q_path,
                       char *payload, int p_size,
                       char **body, int *b_size, char **ctype);
+xs_str *timeline_to_rss(snac *user, const xs_list *timeline, char *title, char *link, char *desc);
 
 int snac_init(const char *_basedir);
 int adduser(const char *uid);
@@ -315,12 +348,15 @@ int oauth_post_handler(const xs_dict *req, const char *q_path,
                        char **body, int *b_size, char **ctype);
 int mastoapi_get_handler(const xs_dict *req, const char *q_path,
                          char **body, int *b_size, char **ctype);
-int mastoapi_delete_handler(const xs_dict *req, const char *q_path,
-                         char **body, int *b_size, char **ctype);
 int mastoapi_post_handler(const xs_dict *req, const char *q_path,
                           const char *payload, int p_size,
                           char **body, int *b_size, char **ctype);
+int mastoapi_delete_handler(const xs_dict *req, const char *q_path,
+                          const char *payload, int p_size,
+                          char **body, int *b_size, char **ctype);
 int mastoapi_put_handler(const xs_dict *req, const char *q_path,
                           const char *payload, int p_size,
                           char **body, int *b_size, char **ctype);
 void mastoapi_purge(void);
+
+void verify_links(snac *user);
