@@ -98,15 +98,14 @@ void xs_httpd_response(FILE *f, int status, xs_dict *headers, xs_str *body, int
 /* sends an httpd response */
 {
     xs *proto;
-    xs_dict *p;
     xs_str *k;
     xs_val *v;
 
     proto = xs_fmt("HTTP/1.1 %d %s", status, status / 100 == 2 ? "OK" : "ERROR");
     fprintf(f, "%s\r\n", proto);
 
-    p = headers;
-    while (xs_dict_iter(&p, &k, &v)) {
+    int c = 0;
+    while (xs_dict_next(headers, &k, &v, &c)) {
         fprintf(f, "%s: %s\r\n", k, v);
     }
 
